@@ -1,49 +1,53 @@
-check_input <- function(traits,phy,sampling_fraction,root_state_weight){
-  if(class(root_state_weight) == "numeric"){
+check_input <- function(traits,phy,sampling_fraction,root_state_weight,is_complete_tree){
+  if(is.numeric(root_state_weight)){
     if(length(root_state_weight) != length(sort(unique(traits)))){
-      stop("you need to have as many elements in root_state_weight as traits")
+      stop("There need to be as many elements in root_state_weight as there are traits.")
     }
     if(length(which(root_state_weight == 1)) != 1){
-      stop("your root_state_weight needs only one 1")
+      stop("The root_state_weight needs only one 1.")
     }
   } else {
-    if(any(root_state_weight == "maddison_weights" | root_state_weight == "equal_weights"|
+    if(any(root_state_weight == "maddison_weights" |
+           root_state_weight == "equal_weights" |
            root_state_weight == "proper_weights") == FALSE){
-      stop("check root_state_weight for a typo or so")
+      stop("The root_state_weight must be any of maddison_weights, equal_weights, or proper_weights.")
     }
   }
   
   if(ape::is.rooted(phy) == FALSE){
-    stop("the tree needs to be rooted")
+    stop("The tree needs to be rooted.")
   }
   
   if(ape::is.binary(phy) == FALSE){
-    stop("the tree needs to be fully resolved")
+    stop("The tree needs to be fully resolved.")
   }
-  if(ape::is.ultrametric(phy) == FALSE){
-    stop("the tree needs to be ultrametric")
+  if(ape::is.ultrametric(phy) == FALSE & is_complete_tree == FALSE){
+    stop("The tree needs to be ultrametric.")
+  }
+  if(any(phy$edge.length == 0)){
+    stop('The tree must have internode distancs that are all larger than 0.')
   }
   
   if(is.matrix(traits)){
     if(length(sampling_fraction) != length(sort(unique(traits[, 1])))){
-      stop("sampling_fraction must have as many elements as traits you have")
+      stop("Sampling_fraction must have as many elements as the number of traits.")
     }
     
     if(all(sort(unique(as.vector(traits))) == sort(unique(traits[, 1]))) == 
         FALSE){
       stop(
-        "Check your trait argument, if you have more than one column, make sure all your states are included in the first column"
+        "Check your trait argument; if you have more than one column, make sure all your states are included in the first column."
       )
     }
   } else{
     if(length(sampling_fraction) != length(sort(unique(traits)))){
-      stop("sampling_fraction must have as many elements as traits you have")
+      stop("Sampling_fraction must have as many elements as the number of traits.")
     }
   }
   
   if(length(sort(unique(as.vector(traits)))) < 2)
   {
-    stop("the trait has only one state")
+    stop("The trait has only one state.")
   }
 }
 
@@ -56,8 +60,8 @@ secsse_loglik_rhs <- function(t,y,parameter){
   mus <- parameter[[2]]
   Q <- parameter[[3]]
   diag(Q) <- 0
-  dE <- mus-(lambdas + mus + Q %*% (rep(1,d))) * Es + lambdas * Es * Es + ( Q %*% Es )
-  dD <- -(lambdas + mus + Q %*% (rep(1,d)))  * Ds + 2 * lambdas * Es * Ds +( Q %*% Ds )
+  dE <- mus - (lambdas + mus + Q %*% (rep(1,d))) * Es + lambdas * Es * Es + ( Q %*% Es )
+  dD <- -(lambdas + mus + Q %*% (rep(1,d))) * Ds + 2 * lambdas * Es * Ds + ( Q %*% Ds )
   return(list(c(dE,dD)))
 }
 
@@ -79,79 +83,85 @@ ode_FORTRAN <- function(
                           ynames = c("SV"), dimens = n_pars, nout = 1, 
                           dllname = "secsse", method = method, ...
   )[,1:(n_vars + 1)]
+  #print(as.numeric(c(probs[1,1],probs[2,c(1,6:9)])));
   return(probs)
 }
 
-build_initStates_time <- function(phy,traits,num_concealed_states,sampling_fraction){ 
-  if(is.matrix(traits)){ ## trait might be a matrix when a species has more than one state
-    traitStates <- sort(unique(traits[,1]))
-  }else{
-    traitStates <- sort(unique(traits))
+build_states <- function(phy,
+                         traits,
+                         num_concealed_states,
+                         sampling_fraction,
+                         is_complete_tree = FALSE,
+                         mus = NULL) {
+  if(!is.matrix(traits)) {
+    traits <- matrix(traits, nrow = length(traits), ncol = 1, byrow = FALSE)
   }
+  if(length(phy$tip.label) != nrow(traits)) {
+    stop("Number of species in the tree must be the same as in the trait file")
+  }
+  traitStates <- sort(unique(traits[,1]))
   
   nb_tip <- ape::Ntip(phy)
   nb_node <- phy$Nnode
-  states <- matrix(ncol = (length(traitStates) * 2 * num_concealed_states),nrow=(nb_tip+nb_node))
-  ly <- ncol(states)
-  d <- ncol(states) / 2
+  ly <- length(traitStates) * 2 * num_concealed_states
+  states <- matrix(ncol = ly, nrow = nb_tip + nb_node)
+  d <- ly / 2
   ## In a example of 3 states, the names of the colums would be like:
   ##
   ## colnames(states) <- c("E0A","E1A","E2A","E0B","E1B","E2B",
   ##                   "D0A","D1A","D2B","D0B","D1B","D2B")
   states[1:nb_tip,] <- 0
-  if(is.matrix(traits)){ ## I repeat the process of state assignation as many times as columns I have
-    for(iv in 1:ncol(traits)){
-      usetraits <- traits[,iv]
-      if(anyNA(usetraits)){
-        nas <- which( is.na(traits))
-        for(iii in 1:length(nas) ){
-          states[nas[iii],] <- c(1 - rep(sampling_fraction,num_concealed_states),
-                                 rep(sampling_fraction,num_concealed_states))
-        }
-      }
-      
-      for(iii in 1:length(traitStates)){ # Initial state probabilities
-        StatesPresents <- d + iii
-        toPlaceOnes <- NULL
-        for(jj in 1:(num_concealed_states-1)){
-          toPlaceOnes <- c(toPlaceOnes, StatesPresents + (length(traitStates)* jj))
-        }
-        toPlaceOnes <- c(StatesPresents,toPlaceOnes)
-        tipSampling <- 1 * sampling_fraction
-        states[which(usetraits == traitStates[iii]),toPlaceOnes] <- tipSampling[iii]
-      }
-      
-      for(iii in 1:nb_tip){
-        states[iii,1:d] <- rep(1 - sampling_fraction,num_concealed_states)
-      }
-    }
-  } else {
-    if(anyNA(traits)){
-      nas <- which( is.na(traits))
-      for(iii in 1:length(nas) ){
-        states[nas[iii],] <- c(1 - rep(sampling_fraction,num_concealed_states),
-                               rep(sampling_fraction,num_concealed_states))
+  #if(is.matrix(traits)){ ## I repeat the process of state assignment as many times as columns I have
+  for(iv in 1:ncol(traits)){
+    usetraits <- traits[,iv]
+    if(anyNA(usetraits)){
+      nas <- which(is.na(traits))
+      for(iii in 1:length(nas)){
+        states[nas[iii],] <- c(1 - rep(sampling_fraction, num_concealed_states),
+                               rep(sampling_fraction, num_concealed_states))
       }
     }
     
     for(iii in 1:length(traitStates)){ # Initial state probabilities
       StatesPresents <- d + iii
-      toPlaceOnes <- NULL
-      for(jj in 1:(num_concealed_states - 1)){
-        toPlaceOnes <- c(toPlaceOnes, StatesPresents + (length(traitStates)* jj))
-      }
-      toPlaceOnes <- c(StatesPresents,toPlaceOnes)
+      #toPlaceOnes <- NULL
+      #for(jj in 1:(num_concealed_states - 1)){
+      #  toPlaceOnes <- c(toPlaceOnes, StatesPresents + (length(traitStates) * jj))
+      #}
+      #toPlaceOnes <- c(StatesPresents, toPlaceOnes)
+      toPlaceOnes <- StatesPresents + length(traitStates) * (0:(num_concealed_states - 1))
       tipSampling <- 1 * sampling_fraction
-      states[which(traits == traitStates[iii]),toPlaceOnes] <- tipSampling[iii]
+      states[which(usetraits == traitStates[iii]),toPlaceOnes] <- tipSampling[iii]
     }
-    
-    for(iii in 1:nb_tip){
-      states[iii,1:d] <- rep(1 - sampling_fraction,num_concealed_states)
+    if(is_complete_tree){
+      extinct_species <- geiger::is.extinct(phy)
+      if(!is.null(extinct_species))
+      {
+        for(i in 1:length(extinct_species)){
+          states[which(phy$tip.label == extinct_species[i]),(d + 1):ly] <- mus
+        }
+      }
+      for(iii in 1:nb_tip){
+        states[iii,1:d] <- 0
+      }
+    } else {
+      for(iii in 1:nb_tip){
+        states[iii,1:d] <- rep(1 - sampling_fraction,num_concealed_states)
+      }
     }
   }
-  
+  return(states)
+}
+
+build_initStates_time <- function(phy,
+                                  traits,
+                                  num_concealed_states,
+                                  sampling_fraction,
+                                  is_complete_tree = FALSE,
+                                  mus = NULL){ 
+  states <- build_states(phy, traits, num_concealed_states, sampling_fraction, is_complete_tree, mus)
   phy$node.label <- NULL
-  split_times <- sort(ape::branching.times(phy), decreasing = F)
+  split_times <- sort(event_times(phy), decreasing = F)
   ances <- as.numeric(names(split_times))
   forTime <- matrix(NA,ncol = 3,nrow = nrow(phy$edge))
   forTime[,1:2] <- phy$edge
@@ -169,7 +179,7 @@ build_initStates_time <- function(phy,traits,num_concealed_states,sampling_fract
       } else {
         timeInterv <- c(0, rootward_age)
       }  
-      forTime[ which(forTime[,2] == desNodes[desIndex]),3] <- timeInterv[2]-timeInterv[1]
+      forTime[which(forTime[,2] == desNodes[desIndex]),3] <- timeInterv[2] - timeInterv[1]
     }
   }
   return(list(
@@ -187,7 +197,8 @@ calThruNodes <- function(
   parameter,
   use_fortran,
   methode,
-  phy
+  phy,
+  func
 ){
   lambdas <- parameter[[1]]
   mus <- parameter[[2]]
@@ -201,23 +212,31 @@ calThruNodes <- function(
   focal <- ances
   desRows <- which(phy$edge[, 1] == focal)
   desNodes <- phy$edge[desRows, 2]
-  
   nodeM <- numeric()
   nodeN <- numeric()
   
   for(desIndex in 1:2){
     y <- states[desNodes[desIndex],]
+    #
     timeInte <- forTime[which(forTime[,2] == desNodes[desIndex]),3]
-    ##  To make the calculation in both lineages
+    ##  To do the calculation in both lineages
     
     if(use_fortran == FALSE){
-      nodeMN <- deSolve::ode(y = y, func = secsse_loglik_rhs,
-                             times = c(0,timeInte), parms = parameter,  rtol = reltol, atol = abstol,
+      nodeMN <- deSolve::ode(y = y,
+                             func = func,
+                             times = c(0,timeInte),
+                             parms = parameter,
+                             rtol = reltol,
+                             atol = abstol,
                              method = methode)
     } else {
-      nodeMN <- ode_FORTRAN(y = y, func = "secsse_runmod",
-                             times = c(0,timeInte), parms = parameter,  rtol = reltol, atol = abstol,
-                             method = methode)
+      nodeMN <- ode_FORTRAN(y = y,
+                            func = func,
+                            times = c(0,timeInte),
+                            parms = parameter,
+                            rtol = reltol,
+                            atol = abstol,
+                            method = methode)
     }
     if(desIndex == 1){
       nodeN <- nodeMN
@@ -246,117 +265,9 @@ calThruNodes <- function(
   newstate <- nodeM[1:d] ## extinction probabilities
   newstate <- c(newstate,mergeBranch)
   states[focal,] <- newstate
-  #print(parameter); print(loglik)
   return(list(states = states,loglik = loglik,mergeBranch = mergeBranch,nodeM = nodeM))
 }
-# calThruNodes <- function(ances,
-#                          states,
-#                          loglik,
-#                          forTime,
-#                          parameter,
-#                          use_fortran,
-#                          methode,
-#                          phy){
-#   
-#   
-#   lambdas <- parameter[[1]]
-#   mus <- parameter[[2]]
-#   
-#   parameter[[3]][is.na(parameter[[3]])] <- 0
-#   Q <- parameter[[3]]
-#   
-#   nb_node <- phy$Nnode
-#   reltol <- 1e-10
-#   abstol <- 1e-16
-#   hmax <- NULL
-#   ly <- ncol(states)
-#   d <- ncol(states) / 2
-#   
-#   focal <- ances
-#   desRows <- which(phy$edge[, 1] == focal)
-#   desNodes <- phy$edge[desRows, 2]
-#   
-#   nodeM <- numeric()
-#   nodeN <- numeric()
-#   
-#   for(desIndex in 1:2){
-#     y <- states[desNodes[desIndex], ]
-#     timeInte <- forTime[which(forTime[, 2] == desNodes[desIndex]), 3]
-#     ##  To make the calculation in both lineages
-#     
-#     if(use_fortran == "Rsolver"){
-#       nodeMN <- deSolve::ode(
-#         y = y,
-#         func = secsse_loglik_rhs,
-#         times = c(0, timeInte),
-#         parms = parameter,
-#         rtol = reltol,
-#         atol = abstol,
-#         hmax = hmax,
-#         method = methode
-#       )
-#       if(desIndex == 1){
-#         nodeN <- nodeMN
-#         
-#       }
-#       
-#       if(desIndex == 2){
-#         nodeM <- nodeMN
-#       }
-#       
-#     }
-#     
-#     
-#     if(use_fortran == "Fortran"){
-#       nodeMN <- ode_FORTRAN(
-#         y = y,
-#         func = "secsse_loglik_rhs",
-#         times = c(0, timeInte),
-#         parms = parameter,
-#         rtol = reltol,
-#         atol = abstol,
-#         hmax = hmax,
-#         method = methode
-#       )
-#       if(desIndex == 1){
-#         nodeN <- nodeMN
-#         
-#       }
-#       
-#       if(desIndex == 2){
-#         nodeM <- nodeMN
-#       }
-#       
-#     }
-#     
-#   }
-#   ## At the node
-#   
-#   nodeM <- as.numeric(nodeM[2, -1])
-#   nodeN <- as.numeric(nodeN[2, -1])
-#   mergeBranch <- NULL
-#   
-#   for(iii in 1:d){
-#     combProb <- 
-#       nodeM[iii + d] * nodeN[iii + d] * lambdas[iii] ## multiplication of probabilities of both branches
-#     mergeBranch <- c(mergeBranch, combProb)
-#   }
-#   
-#   sumD <- sum(mergeBranch)
-#   mergeBranch <- mergeBranch / sumD
-#   
-#   newstate <- nodeM[1:d] ## extinction probabilities
-#   newstate <- c(newstate, mergeBranch)
-#   
-#   states[focal, ] <- newstate
-#   loglik <- loglik + log(sumD)
-#   return(list(
-#     states = states,
-#     loglik = loglik,
-#     mergeBranch = mergeBranch,
-#     nodeM = nodeM
-#   ))
-# }
+
 #' @importFrom foreach foreach
 #' @importFrom doParallel registerDoParallel
 #' @importFrom foreach %dopar%
@@ -368,7 +279,8 @@ doParalThing <- function(take_ancesSub,
                          parameter,
                          use_fortran,
                          methode,
-                         phy
+                         phy,
+                         func
 ){
   #cl <- makeCluster(2)
   #registerDoParallel(cl)
@@ -389,15 +301,20 @@ doParalThing <- function(take_ancesSub,
                                  .export = c(
                                    "secsse_loglik",
                                    "ode_FORTRAN",
-                                   #"phy",
-                                   #"methode",
                                    "calThruNodes"
-                                   #,"use_fortran")) %dopar% {
                                    )) %dopar% { 
                                      ancesSub <- take_ancesSub[[ii]]
                                      for(i in 1:length(ancesSub)){
                                        calcul <- 
-                                         calThruNodes(ancesSub[i], states, loglik, forTime, parameter, use_fortran = use_fortran,methode = methode, phy = phy)
+                                         calThruNodes(ancesSub[i],
+                                                      states,
+                                                      loglik,
+                                                      forTime,
+                                                      parameter,
+                                                      use_fortran = use_fortran,
+                                                      methode = methode,
+                                                      phy = phy,
+                                                      func = func)
                                        loglik <- calcul$loglik
                                        states <- calcul$states
                                      }
@@ -407,128 +324,18 @@ doParalThing <- function(take_ancesSub,
 }
 
 build_initStates_time_bigtree <- 
-  function(phy, traits, num_concealed_states, sampling_fraction){
-    if(is.matrix(traits)){
-      ## trait might be a matrix when a species has more than one state
-      traitStates <- sort(unique(traits[, 1]))
-    } else{
-      traitStates <- sort(unique(traits))
-    }
-    
-    nb_tip <- ape::Ntip(phy)
-    nb_node <- phy$Nnode
-    states <- 
-      matrix(ncol = (length(traitStates) * 2 * num_concealed_states),
-             nrow = (nb_tip + nb_node))
-    
-    
-    ####
-    ly <- ncol(states)
-    d <- ncol(states) / 2
-    ## In a example of 3 states, the names of the colums would be like:
-    ##
-    ## colnames(states) <- c("E0A","E1A","E2A","E0B","E1B","E2B",
-    ##                   "D0A","D1A","D2B","D0B","D1B","D2B")
-    
-    states[1:nb_tip,] <- 0
-    
-    if(is.matrix(traits)){
-      ## I repeat the process of state assignation as many times as columns I have
-      for(iv in 1:ncol(traits)){
-        usetraits <- traits[, iv]
-        if(anyNA(usetraits)){
-          nas <- which(is.na(traits))
-          for(iii in 1:length(nas)){
-            states[nas[iii],] <- c(1 - rep(sampling_fraction, num_concealed_states),
-                                   rep(sampling_fraction, num_concealed_states))
-          }
-        }
-        
-        for(iii in 1:length(traitStates)){
-          # Initial state probabilities
-          StatesPresents <- d + iii
-          toPlaceOnes <- NULL
-          for(jj in 1:(num_concealed_states - 1)){
-            toPlaceOnes <- 
-              c(toPlaceOnes, StatesPresents + (length(traitStates) * jj))
-          }
-          toPlaceOnes <- c(StatesPresents, toPlaceOnes)
-          tipSampling <- 1 * sampling_fraction
-          states[which(usetraits == traitStates[iii]), toPlaceOnes] <- 
-            tipSampling[iii]
-        }
-        
-        for(iii in 1:nb_tip){
-          states[iii, 1:d] <- rep(1 - sampling_fraction, num_concealed_states)
-        }
-        
-      }
-    } else {
-      if(anyNA(traits)){
-        nas <- which(is.na(traits))
-        for(iii in 1:length(nas)){
-          states[nas[iii],] <- c(1 - rep(sampling_fraction, num_concealed_states),
-                                 rep(sampling_fraction, num_concealed_states))
-        }
-      }
-      
-      for(iii in 1:length(traitStates)){
-        # Initial state probabilities
-        StatesPresents <- d + iii
-        toPlaceOnes <- NULL
-        for(jj in 1:(num_concealed_states - 1)){
-          toPlaceOnes <- 
-            c(toPlaceOnes, StatesPresents + (length(traitStates) * jj))
-        }
-        toPlaceOnes <- c(StatesPresents, toPlaceOnes)
-        tipSampling <- 1 * sampling_fraction
-        states[which(traits == traitStates[iii]), toPlaceOnes] <- 
-          tipSampling[iii]
-      }
-
-      for(iii in 1:nb_tip){
-        states[iii, 1:d] <- rep(1 - sampling_fraction, num_concealed_states)
-      }
-    }
-    
-    phy$node.label <- NULL
-    split_times <- sort(ape::branching.times(phy), decreasing = F)
-    ances <- as.numeric(names(split_times))
-    
-    forTime <- matrix(NA, ncol = 3, nrow = nrow(phy$edge))
-    forTime[, 1:2] <- phy$edge
-    
-    for(ab in 1:length(ances)){
-      focalTime <- ances[ab]
-      desRows <- which(phy$edge[, 1] == focalTime)
-      desNodes <- phy$edge[desRows, 2]
-      
-      rootward_age <- 
-        split_times[which(names(split_times) == focalTime)]
-      
-      for(desIndex in 1:2){
-        ## to Find the time which the integration will be done in
-        if(any(desNodes[desIndex] == names(split_times))){
-          tipward_age <- 
-            split_times[which(names(split_times) == desNodes[desIndex])]
-          timeInterv <- c(tipward_age, rootward_age)
-        } else {
-          timeInterv <- c(0, rootward_age)
-        }
-        
-        forTime [which(forTime[, 2] == desNodes[desIndex]), 3] <- 
-          timeInterv[2] - timeInterv[1]
-      }
-    }
-    reltol <- 1e-12
-    abstol <- 1e-16
+  function(phy, traits, num_concealed_states, sampling_fraction, is_complete_tree = FALSE, mus = NULL) {
+    initStates_list <- build_initStates_time(phy,traits,num_concealed_states,sampling_fraction,is_complete_tree,mus) 
+    states <- initStates_list$states
+    ances <- initStates_list$ances
+    forTime <- initStates_list$forTime
     
     phySplit <- phy
     phySplit$node.label <- NULL
     nspp <- length(phySplit$tip.label)
     
     split_times <- 
-      sort(ape::branching.times(phySplit), decreasing = T)
+      sort(event_times(phySplit), decreasing = T)
     interNodes <- as.numeric(names(split_times))
     
     formattedtree <- apTreeshape::as.treeshape(phySplit)
@@ -551,7 +358,7 @@ build_initStates_time_bigtree <-
     sub2 <- as.numeric(phylobase::children(phy2,NodeOptimSplit)[2])
     jointSubs <- NodeOptimSplit
     
-    cat("The best split is a subtree which has branches of size:",
+    cat("The best split is into two subtrees with",
         length(geiger::tips(phy, sub1)),"and",length(geiger::tips(phy, sub2)),"tips \n")    
     
     descenSub1 <- sort(phylobase::descendants(phy2, sub1, type = "all"))
@@ -614,10 +421,12 @@ build_initStates_time_bigtree <-
 #' @param root_state_weight the method to weigh the states:"maddison_weights","proper_weights"(default) or "equal_weights". It can also be specified the root state:the vector c(1,0,0) indicates state 1 was the root state.
 #' @param sampling_fraction vector that states the sampling proportion per trait state. It must have as many elements as trait states.
 #' @param run_parallel should the routine to run in parallel be called?
-#' @param setting_calculation argument used internally to speed up calculation. It should be leave blank (default : setting_calculation = NULL)
+#' @param setting_calculation argument used internally to speed up calculation. It should be left blank (default : setting_calculation = NULL)
 #' @param setting_parallel argument used internally to set a parallel calculation. It should be left blank (default : setting_parallel = NULL)
 #' @param see_ancestral_states should the ancestral states be shown? Deafault FALSE
 #' @param loglik_penalty the size of the penalty for all parameters; default is 0 (no penalty)
+#' @param is_complete_tree whether or not a tree with all its extinct species is provided
+#' @param func function to be used in solving the ODE system Currently only for testing purposes.
 #' @note To run in parallel it is needed to load the following libraries when windows: apTreeshape, doparallel and foreach. When unix, it requires: apTreeshape, doparallel, foreach and doMC
 #' @return The loglikelihood of the data given the parameters
 #' @examples
@@ -642,7 +451,7 @@ build_initStates_time_bigtree <-
 #' drill[[2]][] <- 0
 #' drill[[3]][,] <- 0.1
 #' diag(drill[[3]]) <- NA
-#' secsse_loglik(parameter=drill,phylotree,traits,num_concealed_states,
+#' secsse_loglik(parameter = drill,phylotree,traits,num_concealed_states,
 #'    use_fortran,methode,cond,root_state_weight,sampling_fraction,see_ancestral_states = FALSE)
 #'
 #' #[1] -113.1018
@@ -658,9 +467,17 @@ secsse_loglik <- function(parameter,
                           sampling_fraction,
                           run_parallel = FALSE,
                           setting_calculation = NULL,
-                          setting_parallel= NULL,
+                          setting_parallel = NULL,
                           see_ancestral_states = FALSE,
-                          loglik_penalty = 0){
+                          loglik_penalty = 0,
+                          is_complete_tree = FALSE,
+                          func = ifelse(is_complete_tree,
+                                        "secsse_runmod_ct",
+                                        ifelse(use_fortran == FALSE,
+                                               secsse_loglik_rhs,
+                                               "secsse_runmod2")
+                                        )
+){
   lambdas <- parameter[[1]]
   mus <- parameter[[2]]
   parameter[[3]][is.na(parameter[[3]])] <- 0
@@ -668,9 +485,9 @@ secsse_loglik <- function(parameter,
 
   if(run_parallel == TRUE){ 
     if(is.null(setting_calculation)){
-      check_input(traits,phy,sampling_fraction,root_state_weight)
+      check_input(traits,phy,sampling_fraction,root_state_weight,is_complete_tree)
       setting_calculation <- 
-        build_initStates_time_bigtree(phy, traits, num_concealed_states, sampling_fraction)
+        build_initStates_time_bigtree(phy, traits, num_concealed_states, sampling_fraction, is_complete_tree, mus)
     }
     
     states <- setting_calculation$states
@@ -692,10 +509,19 @@ secsse_loglik <- function(parameter,
     take_ancesSub <- list(ancesSub1, ancesSub2)
     
     if(is.null(setting_parallel)){
-      cl <- parallel::makeCluster(2)
-      doParallel::registerDoParallel(cl)
+      if(.Platform$OS.type == "windows"){
+        cl <- parallel::makeCluster(2)
+        doParallel::registerDoParallel(cl)
+        # pass libPath to workers
+        # see https://stackoverflow.com/questions/6412459/how-to-specify-the-location-of-r-packages-in-foreach-packages-pkg-do
+        # https://gitlab.com/CarlBrunius/MUVR/-/issues/11
+        parallel::clusterCall(cl, function(x) .libPaths(x), .libPaths())
+        on.exit(parallel::stopCluster(cl))
+      }
+      if(.Platform$OS.type == "unix"){
+        doMC::registerDoMC(2)
+      } 
     }
-    
     statesNEW <- doParalThing(take_ancesSub,
                               states,
                               loglik,
@@ -703,9 +529,8 @@ secsse_loglik <- function(parameter,
                               parameter,
                               use_fortran,
                               methode,
-                              phy
-    )
-    
+                              phy,
+                              func)
     comingfromSub1 <- statesNEW[[1]][[1]]
     comingfromSub2 <- statesNEW[[2]] [[1]]
     loglik <- statesNEW[[1]][[2]] + statesNEW[[2]][[2]]
@@ -720,16 +545,22 @@ secsse_loglik <- function(parameter,
     
     for(i in 1:length(ancesRest)){
       calcul <- 
-        calThruNodes(ancesRest[i], states, loglik, forTime, parameter, use_fortran = use_fortran,methode=methode, phy= phy)
+        calThruNodes(ancesRest[i],
+                     states,
+                     loglik,
+                     forTime,
+                     parameter,
+                     use_fortran = use_fortran,
+                     methode = methode, 
+                     phy = phy,
+                     func = func)
       states <- calcul$states
       loglik <- calcul$loglik
-      
     }
   } else {
-    
     if(is.null(setting_calculation)){
-      check_input(traits,phy,sampling_fraction,root_state_weight)
-      setting_calculation <- build_initStates_time(phy,traits,num_concealed_states,sampling_fraction)
+      check_input(traits,phy,sampling_fraction,root_state_weight,is_complete_tree)
+      setting_calculation <- build_initStates_time(phy,traits,num_concealed_states,sampling_fraction,is_complete_tree,mus)
     } 
     
     states <- setting_calculation$states
@@ -748,9 +579,19 @@ secsse_loglik <- function(parameter,
     d <- ncol(states) / 2
     
     for(i in 1:length(ances)){
-      calcul <- calThruNodes(ances[i],states,loglik,forTime,parameter,use_fortran = use_fortran,methode=methode, phy=phy)
+      calcul <- calThruNodes(ances[i],
+                             states,
+                             loglik,
+                             forTime,
+                             parameter,
+                             use_fortran = use_fortran,
+                             methode = methode,
+                             phy = phy,
+                             func = func)
       states <- calcul$states
       loglik <- calcul$loglik
+      if(is.na(loglik)) {stop('NA encountered in loglik')}
+      
       nodeN <- calcul$nodeN
     }
   }
@@ -759,7 +600,7 @@ secsse_loglik <- function(parameter,
   nodeM <- calcul$nodeM
   ## At the root
   mergeBranch2 <- (mergeBranch)
-  if(class(root_state_weight) == "numeric"){
+  if(is.numeric(root_state_weight)){
     giveWeights <- root_state_weight/num_concealed_states
     weightStates <- rep(giveWeights,num_concealed_states)
   } else {
